﻿using System;
using Windows.Foundation;
using Windows.UI.Xaml.Controls;

namespace WebRVDemo
{
    public class RVWebView 
    {
        private readonly Panel webRVContainer;
        private readonly WebView webView;

        public event TypedEventHandler<WebView, WebRVEventArgs> OnInit;
        public event TypedEventHandler<WebView, WebRVEventArgs> OnClose;
        public event TypedEventHandler<WebView, WebRVEventArgs> OnOpen;
        public event TypedEventHandler<WebView, WebRVEventArgs> OnReady;
        public event TypedEventHandler<WebView, WebRVEventArgs> OnCompletion;
        public event TypedEventHandler<WebView, WebRVEventArgs> OnNoCampaigns;
        public event TypedEventHandler<WebView, WebRVEventArgs> OnError;

        public RVWebView(Panel webRVContainer, string appID, string appUserId)
        {
            // The webRVContainer stores the WebView.
            this.webRVContainer = webRVContainer;

            // The WebView stretches to fit its container.   
            webView = new WebView(WebViewExecutionMode.SameThread)
            {
                VerticalAlignment = Windows.UI.Xaml.VerticalAlignment.Stretch,
                HorizontalAlignment = Windows.UI.Xaml.HorizontalAlignment.Stretch,
                DefaultBackgroundColor = Windows.UI.Color.FromArgb(0, 0, 0, 255),
                AllowDrop = false
            };          
            webView.NavigationCompleted += NavigationCompleted;
            webView.ScriptNotify += ScriptNotify;

            StartSdk(appID, appUserId);
        }

        // Running functions defined in the SDK HTML page
        public async void ShowRV()
        {
            AddRVToContainer();

            string[] args = new string[] { "showRV()" };
            await webView.InvokeScriptAsync("eval", args);
        }

        public async void UpdateID(string appID)
        {
            string[] args = new string[] { "updateRV('" + appID + "')" };
            await webView.InvokeScriptAsync("eval", args);
        }

        // Loading the SDK by navigating to the HTML page (must be defined in the ApplicationContentUriRules)
        private void StartSdk(string appID, string appUserId)
        {
            string adkURL = "https://static.ultra-rv.com/ms/ms_app_sample.html";
            string applicationKey = "applicationKey=" + appID;
            string applicationUserId = "applicationUserId=" + appUserId;
            string storeId = "storeId=" + Windows.System.UserProfile.AdvertisingManager.AdvertisingId;

            webView.Navigate(new Uri(adkURL + "?" + applicationKey + "&" + applicationUserId +"&" + storeId));
        }

        private void NavigationCompleted(WebView sender, WebViewNavigationCompletedEventArgs args)
        {
            if (args.IsSuccess)
                OnInit?.Invoke(sender, new WebRVEventArgs("SDK initialization completed"));
            else
                OnError?.Invoke(sender, new WebRVEventArgs("SDK initialization failed with status: " + args.WebErrorStatus));
        }

        // Translate JS notifications to c# events
        private void ScriptNotify(object sender, NotifyEventArgs e)
        {
            string notification = e.Value;
            System.Diagnostics.Debug.WriteLine(string.Format("ScriptNotify: {0}", notification));
            if (notification == "CLOSE")
            {
                OnClose?.Invoke((WebView)sender, new WebRVEventArgs("CLOSE Event Received: Closing"));
                RemoveRVFromContainer();
            }
            else if (notification == "OPEN")
            {
                OnOpen?.Invoke((WebView)sender, new WebRVEventArgs("OPEN Event Received: Ad on screen"));
            }
            else if (notification == "READY")
            {
                OnReady?.Invoke((WebView)sender, new WebRVEventArgs("READY Event Received: Ad available"));
            }
            else if (notification == "COMPLETION")
            {
                OnCompletion?.Invoke((WebView)sender, new WebRVEventArgs("COMPLETION Event Received: Reward received"));
            }
            else if (notification == "NO_CAMPAIGNS")
            {
                OnNoCampaigns?.Invoke((WebView)sender, new WebRVEventArgs("NO_CAMPAIGNS Event Received: No ads available"));
                RemoveRVFromContainer();
            }
            else if (notification.Contains("ERROR"))
            {
                OnError?.Invoke((WebView)sender, new WebRVEventArgs("Script error"));
                RemoveRVFromContainer();
            }
            else
            {
                OnNoCampaigns?.Invoke((WebView)sender, new WebRVEventArgs(string.Format("Unknown event : {0}", notification)));
            }
        }


        private void AddRVToContainer()
        {
            webView.Width = webRVContainer.ActualWidth;
            webView.Height = webRVContainer.ActualHeight;
            if (!webRVContainer.Children.Contains(webView))
                webRVContainer.Children.Add(webView);
        }

        private void RemoveRVFromContainer()
        {
            if (webRVContainer.Children.Contains(webView))
                webRVContainer.Children.Remove(webView);
        }
    }

    public class WebRVEventArgs : EventArgs
    {
        public WebRVEventArgs(string message)
        {
            Message = message;
        }
        public string Message { get; set; }
    }
}
